#include "notepad.h"
#include "ui_notepad.h"
#include "exitapp.h"

Notepad::Notepad(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::Notepad)
{
    ui->setupUi(this);
    this->setWindowTitle(tr("Текстовый редактор"));
    ui->newFileAction->setIcon(QPixmap(":/images/create.jpg"));
    ui->openFileAction->setIcon(QPixmap(":/images/open.jpg"));
    ui->saveFileAction->setIcon(QPixmap(":/images/save.jpg"));
    ui->saveAllAction->setIcon(QPixmap(":/images/saveAll.jpg"));

    ui->menubar->setStyleSheet("color: rgb(255, 255, 255);");

    bar = new QToolBar(this);
    bar->addAction(ui->newFileAction);
    bar->addAction(ui->openFileAction);
    bar->addAction(ui->saveFileAction);
    bar->addAction(ui->saveAllAction);
    addToolBar(Qt::TopToolBarArea, bar);

    ui->newFileAction->setShortcut(QKeySequence("CTRL+N"));
    ui->openFileAction->setShortcut(QKeySequence("CTRL+O"));
    ui->saveFileAction->setShortcut(QKeySequence("CTRL+S"));
    ui->saveAllAction->setShortcut(QKeySequence("CTRL+SHIFT+S"));
    ui->saveFileAsAction->setShortcut(QKeySequence("CTRL+ALT+S"));
    ui->closeFileAction->setShortcut(QKeySequence("CTRL+W"));
    ui->closeAllAction->setShortcut(QKeySequence("CTRL+SHIFT+W"));
    ui->exitAction->setShortcut(QKeySequence("CTRL+Q"));

    ui->selectAllAction->setShortcut(QKeySequence("CTRL+A"));
    ui->copyAction->setShortcut(QKeySequence("CTRL+C"));
    ui->pasteAction->setShortcut(QKeySequence("CTRL+V"));
    ui->cutAction->setShortcut(QKeySequence("CTRL+X"));

    createDockWidgets();
}

Notepad::~Notepad()
{
    delete ui;
}

void Notepad::createDockWidgets()
{
    listView=new QListView(this);
    listView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    listView->setModel(stringList);
    listView->setMinimumWidth(80);

    openDocs=new QDockWidget(tr("Открытые файлы"),this);
    openDocs->setAllowedAreas(Qt::RightDockWidgetArea);
    addDockWidget(Qt::RightDockWidgetArea,openDocs);
    openDocs->setWidget(listView);

    openDocs->setStyleSheet("QDockWidget {color:white; background: rgb(22, 22, 22);}");

    listView->setStyleSheet("QListView {color:white; background: rgb(22, 22, 22);} "
                            "QListView {selection-background-color:  rgb(255, 188, 188);}");
    connect(listView,SIGNAL(clicked(QModelIndex)),this,SLOT(currentTab(QModelIndex)));
    openDocs->hide();


    fileSystem = new QFileSystemModel(this);
    fileSystem->setRootPath(QDir::currentPath());
    fileExplorer=new QDockWidget(tr("Проводник"),this);
    fileExplorer->setAllowedAreas(Qt::LeftDockWidgetArea);
    addDockWidget(Qt::LeftDockWidgetArea,fileExplorer);
    QTreeView *treeView = new QTreeView(this);
    treeView->setModel(fileSystem);
    treeView->setMinimumWidth(80);
    fileExplorer->setWidget(treeView);

    fileExplorer->setStyleSheet("background-color: rgb(22, 22, 22); color: white;");
    treeView->setStyleSheet("QHeaderView::section { color: white; background-color: rgb(35, 35, 35); gridline-color: rgb(35, 35, 35); }");

    fileExplorer->hide();
    connect(treeView, SIGNAL(clicked(QModelIndex)), this, SLOT(loadCurrentFile(QModelIndex)));
}

void Notepad::closeEvent(QCloseEvent *closeEvent)
{
    QStringList listNoSaveFiles;
    for(int i=0;i<ui->mainWidget->count();++i){
        ui->mainWidget->setCurrentIndex(i);
        if(ui->mainWidget->currentWidget()->property("path")==QVariant())
            listNoSaveFiles.push_back(ui->mainWidget->tabText(ui->mainWidget->currentIndex()));
        else{
            QFile file(ui->mainWidget->currentWidget()->property("path").toString());
            if(file.open(QIODevice::ReadOnly)){
                QString text=file.readAll();
                QTextEdit* page=static_cast<QTextEdit*>(ui->mainWidget->currentWidget());
                if(page->toPlainText()!=text) listNoSaveFiles.push_back(ui->mainWidget->currentWidget()->property("path").toString());
            }
            else QMessageBox::critical(this, tr("Ошибка"), tr("Не получается открыть файл: ")
                                       + ui->mainWidget->tabText(ui->mainWidget->currentIndex()) + "\n" + file.errorString());
        }
    }
    if(listNoSaveFiles.count()!=0){
        ExitApp* exitApp_window=new ExitApp(&listNoSaveFiles);
        int resultExitWindow=exitApp_window->exec();
        if(resultExitWindow==1){
            QWidget* widget=ui->mainWidget->currentWidget();
            for(int a=0;a<ui->mainWidget->count();++a){
                ui->mainWidget->setCurrentIndex(a);
                on_saveFileAction_triggered();
            }
            ui->mainWidget->setCurrentWidget(widget);
        }
        if (resultExitWindow==1 || resultExitWindow==2) closeEvent->accept();
        else closeEvent->ignore();
    }
}

void Notepad::changeDoc(){
    if (!(ui->mainWidget->tabText(ui->mainWidget->currentIndex())).contains("*")){
        ui->mainWidget->setTabText(ui->mainWidget->currentIndex(), ui->mainWidget->tabText(ui->mainWidget->currentIndex()) + "*");
    }
    updateOpenDocs();
}

void Notepad::updateOpenDocs()
{
    if(stringList!=Q_NULLPTR){
        delete stringList;
        stringList=new QStringListModel(this);
        QStringList list;
        for(int i = 0;i < ui->mainWidget->count(); ++i){
            list.push_back(ui->mainWidget->tabText(i));
        }
        stringList->setStringList(list);
        listView->setModel(stringList);
        listView->reset();
    }
}

bool Notepad::dialogWindow()
{
    QMessageBox message;
    QPushButton *save = message.addButton(tr("Сохранить"), QMessageBox::ActionRole);
    QPushButton *cancel = message.addButton(tr("Отмена"), QMessageBox::ActionRole);
    QPushButton *notSave = message.addButton(tr("Не сохранять"), QMessageBox::ActionRole);
    save->setStyleSheet("QPushButton{ color: white; background-color:  rgb(80, 80, 80); }"
                        "QPushButton:pressed{ color: white; background-color: rgb(20, 20, 20); }");
    notSave->setStyleSheet("QPushButton{ color: white; background-color:  rgb(80, 80, 80); }"
                           "QPushButton:pressed{ color: white; background-color: rgb(20, 20, 20); }");
    cancel->setStyleSheet("QPushButton{ color: white; background-color:  rgb(80, 80, 80); }"
                          "QPushButton:pressed{ color: white; background-color: rgb(20, 20, 20); }");
    Q_UNUSED(notSave);
    message.setWindowTitle(tr("Сохранение изменений"));
    message.setIcon(QMessageBox::Warning);
    message.setStyleSheet("background-color: rgb(118, 118, 118); color: white;");
    message.setText(tr("Файл ") + ui->mainWidget->tabText(ui->mainWidget->currentIndex()) + tr(" не сохранён."));
    message.setInformativeText(tr("Хотите сохранить его?"));
    message.exec();
    if ((QPushButton*)(message.clickedButton())==save) {
        on_saveFileAction_triggered();
    }
    if ((QPushButton*)(message.clickedButton())==cancel) {
        return false;
    }
    return true;
}

void Notepad::loadCurrentFile(QModelIndex index)
{
    QFile file(fileSystem->filePath(index));
    if (file.open(QIODevice::ReadOnly)) {
        bool openingFile=false;
        for(int i = 0; i < ui->mainWidget->count(); ++i) {
            if(ui->mainWidget->tabText(i)==fileSystem->fileName(index)) openingFile=true;
        }
        if(!openingFile){
           QString textFile=file.readAll();
           CodeEditor* editor = new CodeEditor(this);
           editor->setProperty("path",fileSystem->filePath(index));
           editor->appendPlainText(textFile);

           QString openedFile=fileSystem->fileName(index);
           QString extension = QFileInfo(openedFile).suffix();
           if (!extension.isEmpty()){
               highlighter = new Highlighter(extension, editor->document());
               if (!highlighter->isSupported())
                   delete highlighter;
           }

           QFileInfo fileInfo(fileSystem->filePath(index));
           ui->mainWidget->addTab(editor,fileInfo.fileName());

           connect(editor,SIGNAL(textChanged()),this,SLOT(changeDoc()));
        }
        else QMessageBox::critical(this, tr("Ошибка"), tr("Файл с таким именем уже открыт"));
    }
    else QMessageBox::critical(this, tr("Ошибка"), tr("Не получается открыть файл: ") +
                               fileSystem->fileName(index) + "\n" + file.errorString());
}

void Notepad::on_newFileAction_triggered()
{
    ++numberOfFiles;
    CodeEditor* editor = new CodeEditor(this);
    ui->mainWidget->addTab(editor, QString("Новый документ ") + QString::number(numberOfFiles));

    updateOpenDocs();
    connect(editor,SIGNAL(textChanged()),this,SLOT(changeDoc()));
}

void Notepad::on_openFileAction_triggered()
{
    QStringList nameFiles=QFileDialog::getOpenFileNames(this,tr("Открыть файл(-ы)"),"","");
    for(auto nameFile:nameFiles){
        QFile file(nameFile);
        if(file.open(QIODevice::ReadOnly)){
            bool openingFile=false;
            for (int i = 0; i < ui->mainWidget->count(); ++i){
                if (ui->mainWidget->tabText(i)==QFileInfo(nameFile).fileName()) openingFile=true;
            }
            if(!openingFile){
               QString textFile=file.readAll();
               CodeEditor* editor = new CodeEditor(this);
               editor->setProperty("path",nameFile);
               editor->appendPlainText(textFile);

               QString extension = QFileInfo(nameFile).suffix();
               if (!extension.isEmpty()){
                   highlighter = new Highlighter(extension, editor->document());
                   if (!(highlighter->isSupported()))
                       delete highlighter;
               }

               QFileInfo fileInfo(nameFile);
               ui->mainWidget->addTab(editor,fileInfo.fileName());

               connect(editor,SIGNAL(textChanged()),this,SLOT(changeDoc()));
            }
            else QMessageBox::critical(this, tr("Ошибка"), tr("Файл с таким именем уже открыт"));
        }
        else QMessageBox::critical(this, tr("Ошибка"), tr("Не получается открыть файл: ") + nameFile + "\n" + file.errorString());
    }

}

void Notepad::on_saveFileAction_triggered()
{
    if(ui->mainWidget->currentWidget()->property("path")!=QVariant()){
        CodeEditor* page = static_cast<CodeEditor*>(ui->mainWidget->currentWidget());
        QFile file(ui->mainWidget->currentWidget()->property("path").toString());
        if(file.open(QIODevice::WriteOnly)) {
            QTextStream stream(&file);
            stream<< page->toPlainText();
        } else {
        QMessageBox::critical(this, tr("Ошибка"), tr("Невозможно сохранить файл: ") +
                              ui->mainWidget->tabText(ui->mainWidget->currentIndex()) + "  \n" + file.errorString());
        }
    } else {
        on_saveFileAsAction_triggered();
    }
}

void Notepad::on_saveFileAsAction_triggered()
{
    QFileDialog fileDialog(this);
    fileDialog.setWindowTitle(tr("Сохранить ")+ui->mainWidget->tabText(ui->mainWidget->currentIndex()).remove("*")+ tr(" как..."));
    fileDialog.setFileMode(QFileDialog::AnyFile);
    QString fileName=QFileDialog::getSaveFileName(this,tr("Сохранить ") + ui->mainWidget->tabText(ui->mainWidget->currentIndex()).remove("*")
                                                  + tr(" как..."), ui->mainWidget->tabText(ui->mainWidget->currentIndex()),tr("Все файлы"));
    QFile file(fileName);
    CodeEditor* page=static_cast<CodeEditor*>(ui->mainWidget->currentWidget());

    if(file.open(QIODevice::WriteOnly)){
        QTextStream stream(&file);
        stream << page->toPlainText();
        QFileInfo fileInfo(fileName);
        ui->mainWidget->setTabText(ui->mainWidget->currentIndex(),fileInfo.fileName());
    } else {
        QMessageBox::critical(this, tr("Ошибка"), tr("Невозможно сохранить файл: ") + fileName + "  \n" + file.errorString());
    }
    updateOpenDocs();
}

void Notepad::on_saveAllAction_triggered()
{
    QWidget* curWidget = ui->mainWidget->currentWidget();
    for(int i = 0; i < ui->mainWidget->count(); ++i){
        ui->mainWidget->setCurrentIndex(i);
        on_saveFileAction_triggered();
    }
    ui->mainWidget->setCurrentWidget(curWidget);
}

bool Notepad::on_closeFileAction_triggered()
{
    bool close=true;
    if (ui->mainWidget->currentWidget()->property("path")==QVariant()){
        close=dialogWindow();
    }
    else {
        QFile file(ui->mainWidget->currentWidget()->property("path").toString());
        if (file.open(QIODevice::ReadOnly)){
            QString text=file.readAll();
            CodeEditor* page=static_cast<CodeEditor*>(ui->mainWidget->currentWidget());
            if (page->toPlainText()!=text) close=dialogWindow();
        }
        QMessageBox::critical(this, tr("Ошибка"), tr("Невозможно сохранить файл: ") +
                              ui->mainWidget->tabText(ui->mainWidget->currentIndex()) +" \n" + file.errorString());
    }
    if (close){
        ui->mainWidget->currentWidget()->close();
        ui->mainWidget->removeTab(ui->mainWidget->currentIndex());
    }
    updateOpenDocs();
    return close;
}

void Notepad::on_closeAllAction_triggered()
{
    int count = ui->mainWidget->count();
    int index = 0;
    for (int i = 0 ; i < count ; i++) {
          ui->mainWidget->setCurrentIndex(index);
          if (!on_closeFileAction_triggered())
              ++index;
     }
}

void Notepad::on_exitAction_triggered()
{
    close();
}

void Notepad::on_cutAction_triggered()
{
    CodeEditor* textEdit =  qobject_cast<CodeEditor*> (ui->mainWidget->currentWidget());
    textEdit->cut();
}

void Notepad::on_copyAction_triggered()
{
    CodeEditor* textEdit =  qobject_cast<CodeEditor*> (ui->mainWidget->currentWidget());
    textEdit->copy();
}

void Notepad::on_pasteAction_triggered()
{
    CodeEditor* textEdit =  qobject_cast<CodeEditor*> (ui->mainWidget->currentWidget());
    textEdit->paste();
}

void Notepad::on_deleteAction_triggered()
{
    CodeEditor* textEdit =  qobject_cast<CodeEditor*> (ui->mainWidget->currentWidget());
    textEdit->clear();
}

void Notepad::on_selectAllAction_triggered()
{
    CodeEditor* textEdit =  qobject_cast<CodeEditor*> (ui->mainWidget->currentWidget());
    textEdit->selectAll();
}

void Notepad::on_fileExplorer_visibilityChanged(bool visible) {
    ui->showFileExplorer->setChecked(visible);
}

void Notepad::on_openDocs_visibilityChanged(bool visible) {
    ui->showOpenDocs->setChecked(visible);
}

void Notepad::on_showFileExplorer_triggered(bool visible)
{
    connect(fileExplorer, SIGNAL(visibilityChanged(bool)), this, SLOT(on_fileExplorer_visibilityChanged(bool)));
    if (visible) {
       fileExplorer->show();
    } else {
       fileExplorer->close();
    }
}

void Notepad::on_showOpenDocs_triggered(bool visible)
{
    connect(openDocs, SIGNAL(visibilityChanged(bool)), this, SLOT(on_openDocs_visibilityChanged(bool)));
    if (visible) {
        stringList = new QStringListModel(this);
        QStringList list;
        for(int i = 0; i < ui->mainWidget->count(); ++i){
            list.push_back(ui->mainWidget->tabText(i));
        }
        stringList->setStringList(list);
        openDocs->show();
    } else {
       openDocs->close();
    }
}

void Notepad::currentTab(QModelIndex index){
    ui->mainWidget->setCurrentIndex(index.row());
}

void Notepad::on_mainWidget_tabCloseRequested(int index)
{
    QWidget* curWidget = ui->mainWidget->currentWidget();
    ui->mainWidget->setCurrentIndex(index);
    on_closeFileAction_triggered();
    ui->mainWidget->setCurrentWidget(curWidget);
}
